init 1 python:
    import copy, random
    random.seed()
    
    _witch = 0
    _healer = 1
    _sword = 3
    
    _up = 0
    _right = 1
    _down = 2
    _left = 3
    
    class Sprite():
        def __init__(self, spritesheet, friendly):
            #renpy.Displayable.__init__(self)
            self.spritesheet = spritesheet
            self.height, self.width = 256, 144
            self.height /= 4
            self.width /= 3
            self.xpos = 450
            self.ypos = 160
            self.move = 0
            self.damage = False
            self.heal = False
            if friendly:
                self.dir = _left
            else:
                self.dir = _right
        
        def image(self):
            cropped = im.Crop(self.spritesheet, self.width, self.height*self.dir, self.width, self.height)
            
            if self.damage:
                self.damage = False
                return Animation(cropped, 0.7, im.MatrixColor(cropped,im.matrix.brightness(0.8)), 0.2, cropped)
            
            if self.heal:
                self.heal = False
                return Animation(cropped, 0.7, im.MatrixColor(im.MatrixColor(cropped,im.matrix.tint(0.5,0.5,1)),im.matrix.brightness(0.8)), 0.2, cropped)
            
            if self.move:
                return anim.Filmstrip(im.Crop(self.spritesheet, 0, self.height*self.dir, self.width*3, self.height),(self.width, self.height), (3,1), 0.2, anim_timebase=True)
            else:
                return cropped
    
    class MonsterSprite(Sprite):
        def image(self):
            return self.spritesheet
    
    class BattleCharacter():
        def __init__(self, name, friendly, spritesheet):
            self.name = name
            self.classname = ""
            self.baseattack = 0
            self.speed = self.tempspeed = 0
            self.health = self.maxhealth = 6
            self.hitchance = self.temphitchance = 0.5
            self.hitbonus = self.temphitbonus = 0.0
            self.stopped = 0
            self.friendly = friendly
            self.hit = ""
            self.miss = ""
            
            self.spells = [ ]
            self.items = [ ]
            self.cancast = [ ]
            self.ability = None
            self.abilitytext = ""
            self.sprite = Sprite(Image(spritesheet), friendly)
        
        def attack(self, target):
            self.sprite.move = 1
            
            if random.random() + self.temphitbonus > target.temphitchance and target.health:
                target.take_damage(self, self.baseattack, True)
                x(fill_names(self.hit, self, target))
            else:
                x(fill_names(self.miss, self, target))
        
        def take_damage(self, attacker, damage, counterable):
            self.health -= damage
            if self.health < 0:
                self.health = 0
            
            self.sprite.damage = True
            renpy.restart_interaction()
        
        def recover_damage(self, damage):
            self.sprite.heal = True
            self.health += damage
            if self.health > self.maxhealth:
                self.health = self.maxhealth
            renpy.restart_interaction()
        
        def show_menu(self):
            renpy.restart_interaction()
            
            ui.layer("overlay")
            ui.window(xfill=False, xpos=1.0, xanchor=1.0, ypos=0.65, yminimum=0, yanchor=1.0, xmargin=20)
            ui.vbox()
            ui.text(self.name)
            ui.textbutton("Attack", clicked=ui.returns(1))
            if self.spells:
                ui.textbutton("Magic", clicked=ui.returns(2))
            if self.items:
                ui.textbutton("Item", clicked=ui.returns(3))
            if self.ability:
                ui.textbutton(self.abilitytext, clicked=ui.returns(4))
            ui.close()
            ui.close()
            
            return ui.interact()
        
        def copy(self):
            newinstance = copy.copy(self)
            newinstance.sprite = copy.copy(self.sprite)
            return newinstance
        
        def normalize(self):
            self.tempspeed = self.speed
            self.temphitchance = self.hitchance
            self.temphitbonus = self.hitbonus
            if self.friendly:
                self.sprite.dir = _left
                self.sprite.xpos = 450
                self.move = 0
    
    
    class Ranger(BattleCharacter):
        def __init__(self, name, friendly, spritesheet):
            BattleCharacter.__init__(self, name, friendly, spritesheet)
            self.classname = "Ranger"
            self.baseattack = 2
            self.health = self.maxhealth = 6
            self.hitchance = 0.3
            self.hitbonus = 0.3
            self.speed = 9
            self.aiming = False
            self.recover = False
        
        def attack(self, target):
            if not self.aiming:
                BattleCharacter.attack(self, target)
                if not self.recover:
                    self.recover = True
            else:
                x(fill_names(self.hit, self, target))
                target.take_damage(self, self.baseattack+4, True)
                self.aiming = False
        
        def take_aim(self, null=None):
            self.aiming = True
            x(fill_names("%x start%s taking aim...", self))
        
        def use_recover(self, null=None):
            target = select_one(True)
            if not target:
                return 0
            target.recover_damage(2)
            
            x(fill_names("%x tended some of %y's wounds in the heat of combat!", self, target))
        
        def show_menu(self):
            renpy.restart_interaction()
            
            ui.layer("overlay")
            ui.window(xfill=False, xpos=1.0, xanchor=1.0, ypos=0.65, yminimum=0, yanchor=1.0, xmargin=20)
            ui.vbox()
            ui.text(self.name)
            ui.textbutton("Attack", clicked=ui.returns(1))
            if self.spells:
                ui.textbutton("Magic", clicked=ui.returns(2))
            if self.items:
                ui.textbutton("Item", clicked=ui.returns(3))
            if self.ability:
                ui.textbutton(self.abilitytext, clicked=ui.returns(4))
            ui.close()
            ui.close()
            
            return ui.interact()
        
        def pick_ability(self):
            choice = 0
            
            while choice < 2:
                ui.window(xfill=False, xpos=1.0, xanchor=1.0, ypos=0.65, yminimum=0, yanchor=1.0, xmargin=20)
                ui.vbox()
                ui.textbutton("Aim", hovered=ui.returns(0), clicked=ui.returns(2))
                ui.textbutton("Recover", hovered=ui.returns(1), clicked=ui.returns(3))
                ui.close()
                choice = ui.interact()
                
                if choice is 0:
                    x("Take the time to aim for a guaranteed hit and higher damage.", interact=False)
                else:
                    x("Heal your allies during combat for every successful blow landed on enemies.", interact=False)
            
            if choice is 2:
                self.ability = self.take_aim
                self.abilitytext = "Aim"
            else:
                self.ability = self.use_recover
                self.abilitytext = "Recover"
    
    
    class Knight(BattleCharacter):
        def __init__(self, name, friendly, spritesheet):
            BattleCharacter.__init__(self, name, friendly, spritesheet)
            self.classname = "Knight"
            self.baseattack = 5
            self.health = self.maxhealth = 10
            self.hitchance = 0.5
            self.hitbonus = 0.2
            self.counter = False
            self.speed = 4
        
        def take_damage(self, attacker, damage, counterable):
            BattleCharacter.take_damage(self, attacker, damage, counterable)
            
            if self.counter and counterable:
                self.attack(attacker)
        
        def defend(self, null=None):
            target = select_one(True)
            target.hitchance += 2.0
            
            x(fill_names("%x valiantly covers %y from any harm!", self, target))
        
        def show_menu(self):
            for i in party:
                if i.hitchance > 2.0:
                    i.hitchance -= 2.0
            renpy.restart_interaction()
            
            ui.layer("overlay")
            ui.window(xfill=False, xpos=1.0, xanchor=1.0, ypos=0.65, yminimum=0, yanchor=1.0, xmargin=20)
            ui.vbox()
            ui.text(self.name)
            ui.textbutton("Attack", clicked=ui.returns(1))
            if self.spells:
                ui.textbutton("Magic", clicked=ui.returns(2))
            if self.items:
                ui.textbutton("Item", clicked=ui.returns(3))
            if self.ability:
                ui.textbutton(self.abilitytext, clicked=ui.returns(4))
            ui.close()
            ui.close()
            
            return ui.interact()
        
        def pick_ability(self):
            choice = 0
            
            while choice < 2:
                ui.window(xfill=False, xpos=1.0, xanchor=1.0, ypos=0.65, yminimum=0, yanchor=1.0, xmargin=20)
                ui.vbox()
                ui.textbutton("Defend", hovered=ui.returns(0), clicked=ui.returns(2))
                ui.textbutton("Counter", hovered=ui.returns(1), clicked=ui.returns(3))
                ui.close()
                choice = ui.interact()
                
                if choice is 0:
                    x("Valiantly cover an ally from all harm for a single turn.", interact=False)
                else:
                    x("Reflexively respond to every successful hit with a blow of your own.", interact=False)
            
            if choice is 2:
                self.ability = self.defend
                self.abilitytext = "Defend"
                self.counter = False
            else:
                self.counter = True
                self.ability = None
    
    
    class Witch(BattleCharacter):
        def __init__(self, name, friendly, spritesheet):
            BattleCharacter.__init__(self, name, friendly, spritesheet)
            self.classname = "Witch"
            self.baseattack = 1
            self.health = self.maxhealth = 4
            self.hitchance = 0.3
            self.hitbonus = 0.1
            self.speed = 6
            self.gambit = False
            self.type = _witch
            
            self.cancast = [ Spark(), Slow(), Paralyze(), Blizzard(), Superheat(), Blessedstrike(), Fireball(), Lightning() ]
        
        def double_cast(self, null=None):
            itemsmagic(self, self.spells)
            itemsmagic(self, self.spells)
        
        def pick_ability(self):
            choice = 0
            
            while choice < 2:
                ui.window(xfill=False, xpos=1.0, xanchor=1.0, ypos=0.65, yminimum=0, yanchor=1.0, xmargin=20)
                ui.vbox()
                ui.textbutton("Doublecast", hovered=ui.returns(0), clicked=ui.returns(2))
                ui.textbutton("Gambit", hovered=ui.returns(1), clicked=ui.returns(3))
                ui.close()
                choice = ui.interact()
                
                if choice is 0:
                    x("Cast two spells in a row!", interact=False)
                else:
                    x("Take a chance; if you're lucky, you'll retain all the materials that would be used by a spell.", interact=False)
            
            if choice is 2:
                self.ability = self.double_cast
                self.abilitytext = "Doublecast"
                self.gambit = False
            else:
                self.gambit = True
                self.ability = None
    
    
    class Healer(Witch):
        def __init__(self, name, friendly, spritesheet):
            Witch.__init__(self, name, friendly, spritesheet)
            self.classname = "Healer"
            self.type = _healer
            
            self.cancast = [ Cure(), Majorheal(), Haste(), Timestop(), Protect(), Shield(), Restore(), Strengthen() ]
    
    
    class Rogue(BattleCharacter):
        def __init__(self, name, friendly, spritesheet):
            BattleCharacter.__init__(self, name, friendly, spritesheet)
            self.classname = "Rogue"
            self.baseattack = 3
            self.health = self.maxhealth = 4
            self.hitchance = 0.6
            self.hitbonus = 0.3
            self.speed = 15
            self.steal = False
            self.aiming = False
        
        def take_aim(self, null=None):
            self.aiming = True
            x(fill_names("%x sneak%s off to attack from behind...", self))
        
        def show_menu(self):
            ui.layer("overlay")
            ui.window(xfill=False, xpos=1.0, xanchor=1.0, ypos=0.65, yminimum=0, yanchor=1.0, xmargin=20)
            ui.vbox()
            ui.text(self.name)
            if self.steal:
                ui.textbutton("Steal", clicked=ui.returns(1))
            else:
                ui.textbutton("Attack", clicked=ui.returns(1))
            if self.spells:
                ui.textbutton("Magic", clicked=ui.returns(2))
            if self.items:
                ui.textbutton("Item", clicked=ui.returns(3))
            if self.ability:
                ui.textbutton(self.abilitytext, clicked=ui.returns(4))
            ui.close()
            ui.close()
            
            return ui.interact()
        
        def attack(self, target):
            if not self.aiming:
                BattleCharacter.attack(self, target)
                
                if self.steal and random.random() > 0.5:
                    if target.items:
                        self.items += [ target.items.pop() ]
                        x(fill_names("%x managed to steal %z from %y!".replace("%z", self.items[-1].name), self, target))
                    else:
                        x(fill_names("%y has nothing to steal!", target))
            else:
                x(fill_names(self.hit, self, target))
                target.take_damage(self, self.baseattack+4, True)
                self.aiming = False
        
        def pick_ability(self):
            choice = 0
            
            while choice < 2:
                ui.window(xfill=False, xpos=1.0, xanchor=1.0, ypos=0.65, yminimum=0, yanchor=1.0, xmargin=20)
                ui.vbox()
                ui.textbutton("Aim", hovered=ui.returns(0), clicked=ui.returns(2))
                ui.textbutton("Steal", hovered=ui.returns(1), clicked=ui.returns(3))
                ui.close()
                choice = ui.interact()
                
                if choice is 0:
                    x("Take the time to sneak up on an enemy for a guaranteed hit and higher damage.", interact=False)
                else:
                    x("Attempt to steal items on an enemy's person while striking.", interact=False)
            
            if choice is 2:
                self.ability = self.take_aim
                self.abilitytext = "Aim"
                self.steal = False
            else:
                self.steal = True
                self.ability = None
    
    class Grunt(BattleCharacter):
        def __init__(self):
            BattleCharacter.__init__(self, "Grunt", False, "orc-grunt.png")
            self.baseattack = 1
            self.health = self.healthmax = 5
            self.hitbonus = 0.1
            self.hitchance = 0.3
            self.speed = 5
            
            self.hit = "The %x hacks at %y with his mighty axe!"
            self.miss = "The %x tries to take a hack at %y, but hits the ground instead."
            
            for i in range(2):
                self.items.append(allitems[random.randrange(len(allitems))])
        
        def act(self):
            def compare(x, y):
                return y.health - x.health
            
            renpy.pause(1.0)
            if random.random() > 0.4:
                tempparty = party[:]
                tempparty.sort(compare)
                self.attack(tempparty[0])
            else:
                self.attack(party[random.randrange(len(party))])
    
    class Orcmage(BattleCharacter):
        def __init__(self):
            BattleCharacter.__init__(self, "Sorceror", False, "orc-mage.png")
            self.health = self.healthmax = 7
            self.speed = 5
            self.hitbonus = 2.0
            self.hitchance = 0.2
            
            self.hit = "The %x hit%s %y right in the face with a magic bolt!"
            
            for i in range(4):
                self.items.append(allitems[random.randrange(len(allitems))])
        
        def act(self):
            def compare(x, y):
                return y.health - x.health
            
            renpy.pause(1.0)
            if random.random() > 0.3:
                tempparty = party[:]
                tempparty.sort(compare)
                self.attack(tempparty[0])
            else:
                for i in party:
                    i.take_damage(self, 2, False)
                
                x(fill_names("%x summon%s a short blizzard, damaging all!", self))
    
    class Nomad(BattleCharacter):
        def __init__(self):
            BattleCharacter.__init__(self, "Nomad", False, "orc-nomad.png")
            self.baseattack = 1
            self.health = self.healthmax = 5
            self.hitbonus = 0.1
            self.hitchance = 0.3
            self.speed = 5
            
            self.hit = "The %x hacks at %y with his blade!"
            self.miss = "The %x tries to take a hack at %y, but hits the ground instead."
            
            for i in range(5):
                self.items.append(allitems[random.randrange(len(allitems))])
        
        def take_damage(self, attacker, damage, counterable):
            BattleCharacter.take_damage(self, attacker, damage, counterable)
            
            if counterable:
                self.attack(attacker)
        
        def act(self):
            def compare(x, y):
                return y.health - x.health
            
            renpy.pause(1.0)
            tempparty = party[:]
            tempparty.sort(compare)
            
            if random.random() > 0.2:
                self.attack(tempparty[0])
            else:
                tempparty[0].stopped = 3
                tempparty[0].take_damage(self, 0, False)
                x(fill_names("%x strike%s %y so hard, they can't move!", self, tempparty[0]))
    
    class Warrior(BattleCharacter):
        def __init__(self):
            BattleCharacter.__init__(self, "Warrior", False, "orc-warrior.png")
            self.baseattack = 2
            self.health = self.healthmax = 12
            self.hitbonus = 0.2
            self.hitchance = 0.4
            self.speed = 2
            
            self.hit = "The %x takes a monstrously strong swing at %y and hits!"
            self.miss = "The %x's blade swings, but comes within inches of %y..."
            
            for i in range(6):
                self.items.append(allitems[random.randrange(len(allitems))])
        
        def act(self):
            def compare(x, y):
                return y.health - x.health
            
            renpy.pause(1.0)
            if random.random() > 0.4:
                tempparty = party[:]
                tempparty.sort(compare)
                self.attack(tempparty[0])
            else:
                self.attack(party[random.randrange(len(party))])
    
    class Monster(BattleCharacter):
        def __init__(self):
            BattleCharacter.__init__(self, "???", False, "forest-monster.png")
            self.sprite = MonsterSprite("forest-monster.png", False)
            self.baseattack = 3
            self.health = self.healthmax = 52
            self.hitbonus = 0.3
            self.hitchance = 0
            self.speed = 2
            
            self.hit = "Something from the darkness attacks %y!"
            self.miss = "..."
            
            self.sprite.ypos = 40
            self.sprite.xpos = 70
        
        def act(self):
            def compare(x, y):
                return y.health - x.health
            
            renpy.pause(1.0)
            if random.random() > 0.4:
                tempparty = party[:]
                tempparty.sort(compare)
                self.attack(tempparty[0])
            else:
                self.attack(party[random.randrange(len(party))])
    
    class Captain(BattleCharacter):
        def __init__(self):
            BattleCharacter.__init__(self, "Captain", False, "orc-captain.png")
            self.baseattack = 3
            self.health = self.healthmax = 36
            self.hitbonus = 1.0
            self.hitchance = 0.4
            self.speed = 2
            self.partywiped = False
            self.sprite.xpos = 200
            
            self.hit = "The %x takes a monstrously strong swing at %y and hits!"
            self.miss = "The %x's blade swings, but comes within inches of %y..."
            
            for i in range(6):
                self.items.append(allitems[random.randrange(len(allitems))])
        
        def act(self):
            def compare(x, y):
                return y.health - x.health
            
            renpy.pause(1.0)
            if random.random() > 0.4:
                tempparty = party[:]
                tempparty.sort(compare)
                self.attack(tempparty[0])
            else:
                self.attack(party[random.randrange(len(party))])
        
        def take_damage(self, attacker, damage, counterable):
            if damage == 100 and not self.partywiped:
                return
            if self.health < self.healthmax/2 and not self.partywiped:
                return
            BattleCharacter.take_damage(self, attacker, damage, counterable)
            
            if counterable:
                if not self.partywiped:
                    self.attack(attacker)
                else:
                    x(fill_names("%x takes a swing at %y, but gets knocked back!", self, attacker))
                    attacker.take_damage(self, 0, True)
    